@@ -39,28 +39,28 @@ void ssh_split_arguments(int argc, char **argv, int *dest_ac, char ***dest_av)
   for (i = 0; i < argc ; i++)
     {
       if ( alloc < temp_ac + 3)
-	{
-	  alloc = temp_ac + 3;
-	  temp_av = ssh_xrealloc(temp_av, alloc*sizeof(char*));
-	}
+        {
+          alloc = temp_ac + 3;
+          temp_av = ssh_xrealloc(temp_av, alloc*sizeof(char*));
+        }
       
       if (argv[i][0] == '-' || argv[i][0] == '+')
-	{
-	  if(argv[i][1] && argv[i][2])
-	    {
-	      temp_av[temp_ac] = ssh_xstrdup(argv[i]);
-	      temp_av[temp_ac][2] = '\0';
-	      temp_av[++temp_ac] = ssh_xstrdup(argv[i] + 2);
-	    }
-	  else
-	    {
-	      temp_av[temp_ac] = ssh_xstrdup(argv[i]);
-	    }
-	}
+        {
+          if(argv[i][1] && argv[i][2])
+            {
+              temp_av[temp_ac] = ssh_xstrdup(argv[i]);
+              temp_av[temp_ac][2] = '\0';
+              temp_av[++temp_ac] = ssh_xstrdup(argv[i] + 2);
+            }
+          else
+            {
+              temp_av[temp_ac] = ssh_xstrdup(argv[i]);
+            }
+        }
       else
-	{
-	  temp_av[temp_ac] = ssh_xstrdup(argv[i]);	  
-	}
+        {
+          temp_av[temp_ac] = ssh_xstrdup(argv[i]);        
+        }
       temp_ac++;
     }
   temp_av[ temp_ac ] = NULL;
@@ -113,6 +113,7 @@ SshConfig ssh_config_init(Boolean client)
 
   config->port = ssh_xstrdup("22");
   config->ciphers = NULL;
+  config->user_conf_dir = ssh_xstrdup(SSH_USER_CONFIG_DIRECTORY);
   config->identity_file = ssh_xstrdup(SSH_IDENTIFICATION_FILE);
   config->authorization_file = ssh_xstrdup(SSH_AUTHORIZATION_FILE);
 
@@ -157,6 +158,7 @@ SshConfig ssh_config_init(Boolean client)
   config->ssh1_path = NULL;
   config->ssh1compatibility = FALSE;  
 #endif /* SSH1_COMPATIBILITY */
+  config->ssh_agent_compat = SSH_AGENT_COMPAT_NONE;
   
   return config;
 }
@@ -196,7 +198,7 @@ void ssh_config_free(SshConfig config)
   if (config->client == FALSE)
     {
       if (config->private_host_key != NULL)
-	ssh_private_key_free(config->private_host_key);
+        ssh_private_key_free(config->private_host_key);
       ssh_xfree(config->public_host_key_blob);
     }
 
@@ -291,15 +293,15 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
   if (strcmp(var, "port") == 0)
     {
       if (num >= 1 && num < 65536)
-	{
-	  ssh_xfree(config->port);
-	  config->port = ssh_xstrdup(val);
-	}
+        {
+          ssh_xfree(config->port);
+          config->port = ssh_xstrdup(val);
+        }
       else
-	{
-	  ssh_warning("Ignoring illegal port number %s", val);
-	  return TRUE;
-	}
+        {
+          ssh_warning("Ignoring illegal port number %s", val);
+          return TRUE;
+        }
       return FALSE;
     }
   
@@ -308,66 +310,73 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
       SSH_DEBUG(3, ("Got config cipherlist \"%s\"", val));
       ssh_xfree(config->ciphers);
       if (strcasecmp(val, "any") == 0)
-	{
-	  int x;
-	  char *hlp1, *hlp2;
-	  
-	  hlp1 = ssh_cipher_get_supported_native();
-	  config->ciphers = ssh_name_list_intersection(SSH_STD_CIPHERS, hlp1);
-	  hlp2 = ssh_cipher_list_exclude(config->ciphers, "none");
-	  ssh_xfree(config->ciphers);
-	  x = strlen(hlp1) + strlen(hlp2) + 2;
-	  config->ciphers = ssh_xmalloc(x);
-	  snprintf(config->ciphers, x, "%s,%s", hlp2, hlp1);
-	  ssh_xfree(hlp1);
-	  ssh_xfree(hlp2);
-	  hlp1 = ssh_cipher_list_canonialize(config->ciphers);
-	  ssh_xfree(config->ciphers);
-	  config->ciphers = hlp1;
-	}
+        {
+          int x;
+          char *hlp1, *hlp2;
+          
+          hlp1 = ssh_cipher_get_supported_native();
+          config->ciphers = ssh_name_list_intersection(SSH_STD_CIPHERS, hlp1);
+          hlp2 = ssh_cipher_list_exclude(config->ciphers, "none");
+          ssh_xfree(config->ciphers);
+          x = strlen(hlp1) + strlen(hlp2) + 2;
+          config->ciphers = ssh_xmalloc(x);
+          snprintf(config->ciphers, x, "%s,%s", hlp2, hlp1);
+          ssh_xfree(hlp1);
+          ssh_xfree(hlp2);
+          hlp1 = ssh_cipher_list_canonialize(config->ciphers);
+          ssh_xfree(config->ciphers);
+          config->ciphers = hlp1;
+        }
       else if (strcasecmp(val, "anycipher") == 0)
-	{
-	  int x;
-	  char *hlp1, *hlp2;
-	  
-	  hlp2 = ssh_cipher_get_supported_native();
-	  hlp1 = ssh_cipher_list_exclude(hlp2, "none");
-	  ssh_xfree(hlp2);
-	  config->ciphers = ssh_name_list_intersection(SSH_STD_CIPHERS, hlp1);
-	  hlp2 = ssh_cipher_list_exclude(config->ciphers, "none");
-	  ssh_xfree(config->ciphers);
-	  x = strlen(hlp1) + strlen(hlp2) + 2;
-	  config->ciphers = ssh_xmalloc(x);
-	  snprintf(config->ciphers, x, "%s,%s", hlp2, hlp1);
-	  ssh_xfree(hlp1);
-	  ssh_xfree(hlp2);
-	  hlp1 = ssh_cipher_list_canonialize(config->ciphers);
-	  ssh_xfree(config->ciphers);
-	  config->ciphers = hlp1;
-	}
+        {
+          int x;
+          char *hlp1, *hlp2;
+          
+          hlp2 = ssh_cipher_get_supported_native();
+          hlp1 = ssh_cipher_list_exclude(hlp2, "none");
+          ssh_xfree(hlp2);
+          config->ciphers = ssh_name_list_intersection(SSH_STD_CIPHERS, hlp1);
+          hlp2 = ssh_cipher_list_exclude(config->ciphers, "none");
+          ssh_xfree(config->ciphers);
+          x = strlen(hlp1) + strlen(hlp2) + 2;
+          config->ciphers = ssh_xmalloc(x);
+          snprintf(config->ciphers, x, "%s,%s", hlp2, hlp1);
+          ssh_xfree(hlp1);
+          ssh_xfree(hlp2);
+          hlp1 = ssh_cipher_list_canonialize(config->ciphers);
+          ssh_xfree(config->ciphers);
+          config->ciphers = hlp1;
+        }
       else if (strcasecmp(val, "anystd") == 0)
-	{
-	  char *hlp = ssh_cipher_get_supported_native();
-	  config->ciphers = ssh_name_list_intersection(hlp, SSH_STD_CIPHERS);
-	  ssh_xfree(hlp);
-	}
+        {
+          char *hlp = ssh_cipher_get_supported_native();
+          config->ciphers = ssh_name_list_intersection(hlp, SSH_STD_CIPHERS);
+          ssh_xfree(hlp);
+        }
       else if (strcasecmp(val, "anystdcipher") == 0)
-	{
-	  char *hlp = ssh_cipher_get_supported_native();
-	  config->ciphers = ssh_name_list_intersection(hlp, SSH_STD_CIPHERS);
-	  ssh_xfree(hlp);
-	  hlp = config->ciphers;
-	  config->ciphers = ssh_cipher_list_exclude(hlp, "none");
-	  ssh_xfree(hlp);
-	}
+        {
+          char *hlp = ssh_cipher_get_supported_native();
+          config->ciphers = ssh_name_list_intersection(hlp, SSH_STD_CIPHERS);
+          ssh_xfree(hlp);
+          hlp = config->ciphers;
+          config->ciphers = ssh_cipher_list_exclude(hlp, "none");
+          ssh_xfree(hlp);
+        }
       else
-	{
-	  config->ciphers = ssh_cipher_list_canonialize(val);
-	}
+        {
+          config->ciphers = ssh_cipher_list_canonialize(val);
+        }
       SSH_DEBUG(3, ("Final cipherlist \"%s\"", config->ciphers));
       return FALSE;
     }
   
+  if (strcmp(var, "userconfigdirectory") == 0)
+    {
+      ssh_xfree(config->user_conf_dir);
+      config->user_conf_dir = ssh_xstrdup(val);
+      return FALSE;
+    }
+
   if (strcmp(var, "identityfile") == 0)
     {
       ssh_xfree(config->identity_file);
@@ -399,7 +408,7 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
     {
       config->verbose_mode = bool;
       if (bool)
-	ssh_debug_set_level_string("2");
+        ssh_debug_set_level_string("2");
       return FALSE;
     }
 
@@ -432,105 +441,131 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
   if (config->client == TRUE)
     {
       if (strcmp(var, "host") == 0)
-	{
-	  ssh_xfree(config->host_to_connect);
-	  config->host_to_connect = ssh_xstrdup(val);
-	  return FALSE;
-	}
+        {
+          ssh_xfree(config->host_to_connect);
+          config->host_to_connect = ssh_xstrdup(val);
+          return FALSE;
+        }
       if (strcmp(var, "user") == 0)
-	{
-	  ssh_xfree(config->login_as_user);
-	  config->login_as_user = ssh_xstrdup(val);
-	  return FALSE;
-	}
+        {
+          ssh_xfree(config->login_as_user);
+          config->login_as_user = ssh_xstrdup(val);
+          return FALSE;
+        }
       if (strcmp(var, "compression") == 0)
-	{
-	  config->compression = bool;
-	  return FALSE;
-	}
+        {
+          config->compression = bool;
+          return FALSE;
+        }
 
       if (strcmp(var, "fallbacktorsh") == 0)
-	{
-	  config->fall_back_to_rsh = bool;
-	  return FALSE;
-	}
+        {
+          config->fall_back_to_rsh = bool;
+          return FALSE;
+        }
   
       if (strcmp(var, "usersh") == 0)
-	{
-	  config->use_rsh = bool;
-	  return FALSE;
-	}
+        {
+          config->use_rsh = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "batchmode") == 0)
-	{
-	  config->batch_mode = bool;
-	  return FALSE;
-	}
+        {
+          config->batch_mode = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "stricthostkeychecking") == 0)
-	{
-	  config->strict_host_key_checking = bool;
-	  return FALSE;
-	}
+        {
+          config->strict_host_key_checking = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "escapechar") == 0)
-	{
-	  ssh_xfree(config->escape_char);
-	  config->escape_char = ssh_xstrdup(val);
-	  return FALSE;
-	}
+        {
+          ssh_xfree(config->escape_char);
+          config->escape_char = ssh_xstrdup(val);
+          return FALSE;
+        }
       
       if (strcmp(var, "passwordprompt") == 0)
-	{
-	  ssh_xfree(config->password_prompt);
-	  config->password_prompt = ssh_xstrdup(val);
-	  return FALSE;
-	}
+        {
+          ssh_xfree(config->password_prompt);
+          config->password_prompt = ssh_xstrdup(val);
+          return FALSE;
+        }
 
       if (strcmp(var, "gobackground") == 0)
-	{
-	  config->go_background = bool;
-	  return FALSE;
-	}
+        {
+          config->go_background = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "usenonpriviledgedport") == 0)
-	{
-	  config->use_nonpriviledged_port = bool;
-	  return FALSE;
-	}
+        {
+          config->use_nonpriviledged_port = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "dontreadstdin") == 0)
-	{
-	  config->dont_read_stdin = bool;
-	  return FALSE;
-	}
+        {
+          config->dont_read_stdin = bool;
+          return FALSE;
+        }
 
       if (strcmp(var, "ssh1path") == 0)
-	{
-	  ssh_xfree(config->ssh1_path);
-	  config->ssh1_path = ssh_xstrdup(val);
-	  return FALSE;
-	}
+        {
+          ssh_xfree(config->ssh1_path);
+          config->ssh1_path = ssh_xstrdup(val);
+          return FALSE;
+        }
+
+      if (strcmp(var, "ssh1agentcompatibility") == 0)
+        {
+          if (strcasecmp(val, "none") == 0)
+            {
+              config->ssh_agent_compat = SSH_AGENT_COMPAT_NONE;
+              return FALSE;
+            }
+          else if (strcasecmp(val, "traditional") == 0)
+            {
+              config->ssh_agent_compat = SSH_AGENT_COMPAT_TRADITIONAL;
+              return FALSE;
+            }
+          else if (strcasecmp(val, "ssh2") == 0)
+            {
+              config->ssh_agent_compat = SSH_AGENT_COMPAT_SSH2;
+              return FALSE;
+            }
+          else
+            {
+              ssh_warning("Bad Ssh1AgentCompatibility definition \"%s\"", 
+                          val);
+              return TRUE;
+            }
+        }
+
 #ifndef SSHDIST_WINDOWS
       if (strcmp(var, "localforward") == 0)
-	{
-	  if(ssh_parse_forward(&(config->local_forwards), val))
-	    {
-	      ssh_warning("Bad LocalForward definition \"%s\"", val);
-	      return TRUE;
-	    }
-	  return FALSE;
-	}
+        {
+          if(ssh_parse_forward(&(config->local_forwards), val))
+            {
+              ssh_warning("Bad LocalForward definition \"%s\"", val);
+              return TRUE;
+            }
+          return FALSE;
+        }
 
       if (strcmp(var, "remoteforward") == 0)
-	{
-	  if(ssh_parse_forward(&(config->remote_forwards), val))
-	    {
-	      ssh_warning("Bad RemoteForward definition \"%s\"", val);
-	      return TRUE;
-	    }
-	  return FALSE;
-	}  
+        {
+          if(ssh_parse_forward(&(config->remote_forwards), val))
+            {
+              ssh_warning("Bad RemoteForward definition \"%s\"", val);
+              return TRUE;
+            }
+          return FALSE;
+        }  
 #endif /* SSHDIST_WINDOWS */
     }
   else
@@ -538,143 +573,143 @@ Boolean ssh_config_set_parameter(SshConfig config, char *var, char *val)
       /* These parameters are only for the server */
       
       if (strcmp(var, "ignorerhosts") == 0)
-	{
-	  config->ignore_rhosts = bool;
-	  return FALSE;
-	}
+        {
+          config->ignore_rhosts = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "permitrootlogin") == 0)
-	{
-	  config->permit_root_login = bool;
-	  return FALSE;
-	}
+        {
+          config->permit_root_login = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "permitemptypasswords") == 0)
-	{
-	  config->permit_empty_passwords = bool;
-	  return FALSE;
-	}
+        {
+          config->permit_empty_passwords = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "strictmodes") == 0)
-	{
-	  config->strict_modes = bool;
-	  return FALSE;
-	}
+        {
+          config->strict_modes = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "printmotd") == 0)
-	{
-	  config->print_motd = bool;
-	  return FALSE;
-	}
+        {
+          config->print_motd = bool;
+          return FALSE;
+        }
       
       if (strcmp(var, "listenaddress") == 0)
-	{
-	  /* XXX some checks here */
-	  ssh_xfree(config->listen_address);
-	  config->listen_address = ssh_xstrdup(val);
-	  return FALSE;
-	}
+        {
+          /* XXX some checks here */
+          ssh_xfree(config->listen_address);
+          config->listen_address = ssh_xstrdup(val);
+          return FALSE;
+        }
       
       if (strcmp(var, "hostkeyfile") == 0)
-	{
-	  ssh_xfree(config->host_key_file);
-	  config->host_key_file = ssh_xstrdup(val);
+        {
+          ssh_xfree(config->host_key_file);
+          config->host_key_file = ssh_xstrdup(val);
 
-	  /* Note: if you specify PublicHostKeyFile first in the config file,
-	     and HostKey after that, and you give it the value of 
-	     SSH_PUBLIC_HOSTKEY (which, at the moment, is the same as 
-	     SSH_HOSTKEY_FILE with ".pub" appended) the value of 
-	     config->public_host_key_file will be config->host_key_file with 
-	     ".pub" appended. This is a minor problem. This kludge here is to 
-	     avoid changing the value of PublicHostKeyFile depending on the 
-	     order in which config parameters lie in the configfile. */
+          /* Note: if you specify PublicHostKeyFile first in the config file,
+             and HostKey after that, and you give it the value of 
+             SSH_PUBLIC_HOSTKEY (which, at the moment, is the same as 
+             SSH_HOSTKEY_FILE with ".pub" appended) the value of 
+             config->public_host_key_file will be config->host_key_file with 
+             ".pub" appended. This is a minor problem. This kludge here is to 
+             avoid changing the value of PublicHostKeyFile depending on the 
+             order in which config parameters lie in the configfile. */
 
-	  if (strcmp(config->public_host_key_file, SSH_PUBLIC_HOSTKEY) == 0)
-	    {
-	      ssh_xfree(config->public_host_key_file);
-	      num = strlen(val) + strlen(".pub") + 1;
-	      config->public_host_key_file = ssh_xcalloc(1, num);
-	      snprintf(config->public_host_key_file, num, "%s.pub", val);
-	    }
-	  return FALSE;
-	}
+          if (strcmp(config->public_host_key_file, SSH_PUBLIC_HOSTKEY) == 0)
+            {
+              ssh_xfree(config->public_host_key_file);
+              num = strlen(val) + strlen(".pub") + 1;
+              config->public_host_key_file = ssh_xcalloc(1, num);
+              snprintf(config->public_host_key_file, num, "%s.pub", val);
+            }
+          return FALSE;
+        }
       
       if (strcmp(var, "publichostkeyfile") == 0)
-	{
-	  ssh_xfree(config->public_host_key_file);
-	  config->public_host_key_file = ssh_xstrdup(val);
-	  return FALSE;
-	}
-	  
+        {
+          ssh_xfree(config->public_host_key_file);
+          config->public_host_key_file = ssh_xstrdup(val);
+          return FALSE;
+        }
+          
       if (strcmp(var, "logingracetime") == 0)
-	{
-	  if (num < 1)
-	    {
-	      ssh_warning("Ignoring illegal login grace time %d",
-			  num);
-	      return TRUE;
-	    }
-	  config->login_grace_time = num;
-	  return FALSE;
-	}
+        {
+          if (num < 1)
+            {
+              ssh_warning("Ignoring illegal login grace time %d",
+                          num);
+              return TRUE;
+            }
+          config->login_grace_time = num;
+          return FALSE;
+        }
       
       if (strcmp(var, "passwordguesses") == 0)
-	{
-	  config->password_guesses = num;
-	  return FALSE;
-	}
+        {
+          config->password_guesses = num;
+          return FALSE;
+        }
 
       if (strcmp(var, "sshd1path") == 0)
-	{
-	  ssh_xfree(config->ssh1_path);
-	  config->ssh1_path = ssh_xstrdup(val);
-	  return FALSE;
-	}
+        {
+          ssh_xfree(config->ssh1_path);
+          config->ssh1_path = ssh_xstrdup(val);
+          return FALSE;
+        }
       
       /* Parse subsystem definitions */
       
       if (strncmp(var, SUBSYSTEM_PREFIX, SUBSYSTEM_PREFIX_LEN) == 0)
-	{
-	  if (strlen(val) < 1)
-	    {
-	      ssh_warning("Missing subsystem path");
-	      return TRUE;
-	    }
-	      
-	  if (config->no_subsystems > 0)
-	    {
-	      for (i = 0; i < config->no_subsystems; i++)
-		if (strcmp(&var[SUBSYSTEM_PREFIX_LEN], 
-			   config->subsystems[i]->name) == 0)
-	          {
-		    ssh_xfree(config->subsystems[i]->path);
-		    config->subsystems[i]->path = ssh_xstrdup(val);
-		    ssh_warning("Multiple definitions for subsystem %s",
-				config->subsystems[i]->name);
-  		    return FALSE; 
-		  }
-	    }
-	  
-	  if (config->subsystems_allocated == 0 || 
-	      (config->no_subsystems + 1) >= config->subsystems_allocated)
-	    {
-	      if (config->subsystems_allocated < 4)
-		config->subsystems_allocated = 4;
-	      else
-		config->subsystems_allocated *= 2;
-	      
-	      config->subsystems = 
-		ssh_xrealloc(config->subsystems,
-			     config->subsystems_allocated * sizeof (*ss));
-	    }
-	  
-	  ss = ssh_xmalloc(sizeof (*ss));
-	  ss->name = ssh_xstrdup(&var[SUBSYSTEM_PREFIX_LEN]);
-	  ss->path = ssh_xstrdup(val);
-	  config->subsystems[config->no_subsystems++] = ss;
-	  	  	  
-	  return FALSE;
-	}
+        {
+          if (strlen(val) < 1)
+            {
+              ssh_warning("Missing subsystem path");
+              return TRUE;
+            }
+              
+          if (config->no_subsystems > 0)
+            {
+              for (i = 0; i < config->no_subsystems; i++)
+                if (strcmp(&var[SUBSYSTEM_PREFIX_LEN], 
+                           config->subsystems[i]->name) == 0)
+                  {
+                    ssh_xfree(config->subsystems[i]->path);
+                    config->subsystems[i]->path = ssh_xstrdup(val);
+                    ssh_warning("Multiple definitions for subsystem %s",
+                                config->subsystems[i]->name);
+                    return FALSE; 
+                  }
+            }
+          
+          if (config->subsystems_allocated == 0 || 
+              (config->no_subsystems + 1) >= config->subsystems_allocated)
+            {
+              if (config->subsystems_allocated < 4)
+                config->subsystems_allocated = 4;
+              else
+                config->subsystems_allocated *= 2;
+              
+              config->subsystems = 
+                ssh_xrealloc(config->subsystems,
+                             config->subsystems_allocated * sizeof (*ss));
+            }
+          
+          ss = ssh_xmalloc(sizeof (*ss));
+          ss->name = ssh_xstrdup(&var[SUBSYSTEM_PREFIX_LEN]);
+          ss->path = ssh_xstrdup(val);
+          config->subsystems[config->no_subsystems++] = ss;
+                          
+          return FALSE;
+        }
       
     }
   ssh_warning("Unrecognized configuration parameter %s", var);
