@@ -14,19 +14,21 @@
   */
 
 /*
- * $Id: nociph.c,v 1.5 1998/07/28 02:34:57 ylo Exp $
+ * $Id: nociph.c,v 1.6 1998/11/04 12:06:52 ylo Exp $
  * $Log: nociph.c,v $
  * $EndLog$
  */
 
 #include "sshincludes.h"
+#include "nociph.h"
 
 void none_transform(void *context, unsigned char *dest,
-		    const unsigned char *src, size_t len)
+                    const unsigned char *src, size_t len,
+                    unsigned char *iv)
 {
   if (src != dest)
     memcpy(dest, src, len);
 }
 
 /* nociph.c */
-		    
+                    
